/* ******************************************** */
/* Program   : IDX003.I                         */
/* Purpose   : Define temp-table and variables  */
/* Parameters: none                             */
/* ******************************************** */

DEF STREAM stXRef.
DEF STREAM stFile.
DEF STREAM stDebug.
DEF STREAM stExport.

/* Temptable storing the lines from XRef file */
DEF TEMP-TABLE ttXRef NO-UNDO
    FIELD LineId         AS INT
    FIELD xProcName      AS CHAR
    FIELD xFileName      AS CHAR
    FIELD xLineNumber    AS INT
    FIELD xRefType       AS CHAR
    FIELD xObjectId      AS CHAR
    FIELD xMisc1         AS CHAR
    FIELD xMisc2         AS CHAR
    FIELD xMisc3         AS CHAR
    FIELD xMisc4         AS CHAR
    FIELD xMisc5         AS CHAR
    INDEX i1 IS PRIMARY UNIQUE LineId
    INDEX i2 xProcName xFileName xLineNumber.

/* Temptable storing the SEARCH lines from the XRef file */
DEF TEMP-TABLE ttSearch NO-UNDO
    FIELD LineId         AS INT
    FIELD Xref-LineId    AS INT
    FIELD SearchType     AS CHAR    /* Can-Find, Find, For-Each */
    FIELD DatabaseName   AS CHAR
    FIELD TableName      AS CHAR
    FIELD UsedIndexes    AS CHAR    /* Comma-separated list */
    FIELD AccessedFields AS CHAR    /* Comma-separated list */
    FIELD IndexScore1    AS DEC 
    FIELD IndexScore2    AS DEC 
    INDEX i1 IS PRIMARY UNIQUE LineId
    INDEX i2 IndexScore1            /* For Browse Sort */
    INDEX i3 IndexScore2.           /* For Browse Sort */

/* Temptable storing the Temptable structure */
DEF TEMP-TABLE ttTempTable NO-UNDO
    FIELD LineId         AS INT
    FIELD TTName         AS CHAR
    FIELD TTDefinition   AS CHAR
    FIELD TTDBReference  AS CHAR
    INDEX i1 IS PRIMARY UNIQUE LineId
    INDEX i2 TTName.

DEF BUFFER bttXRef FOR ttXRef.

/* General Purpose Working Variables */
DEF VAR cXY                 AS CHAR NO-UNDO.
DEF VAR cFieldList          AS CHAR NO-UNDO.
DEF VAR iLooper             AS INT  NO-UNDO.
DEF VAR iRunner             AS INT  NO-UNDO.
DEF VAR iSearchStats        AS INT  NO-UNDO.
DEF VAR iWorker             AS INT  NO-UNDO.
DEF VAR lVoid               AS LOG  NO-UNDO.

DEF VAR cXRefDatabaseName   AS CHAR NO-UNDO.
DEF VAR cXRefTableName      AS CHAR NO-UNDO.
DEF VAR cXRefIndexName      AS CHAR NO-UNDO.

/* Temptables LineId */
DEF VAR iXrefLineId         AS INT  NO-UNDO INIT 1.
DEF VAR iSearchLineId       AS INT  NO-UNDO INIT 1.
DEF VAR iIndexLineId        AS INT  NO-UNDO INIT 1.
DEF VAR iTempTableLineId    AS INT  NO-UNDO INIT 1.

/* Wordking variables for XRef Analyse */        
DEF VAR lTableTagExist      AS LOG  NO-UNDO.
DEF VAR lTableTagMeet       AS LOG  NO-UNDO.    

/* Working variables for ttIndex */
DEF VAR iFieldSeq           AS INT  NO-UNDO.
DEF VAR lIsUnique           AS LOG  NO-UNDO.

/* Working variables for Score calculation */
DEF VAR cFldMask            AS CHAR NO-UNDO.
DEF VAR dScore              AS DEC  NO-UNDO.
DEF VAR iNumIndexedFields   AS INT  NO-UNDO.

/* Working variables for Temptable analysis */
DEF VAR cIndexName          AS CHAR NO-UNDO.
DEF VAR cLine               AS CHAR NO-UNDO.
DEF VAR cVoid               AS CHAR NO-UNDO.
DEF VAR cFileName           AS CHAR NO-UNDO.
DEF VAR cProcName           AS CHAR NO-UNDO.
DEF VAR cTableName          AS CHAR NO-UNDO.
DEF VAR iLineNumber-from    AS INT  NO-UNDO.
DEF VAR iLineNumber-to      AS INT  NO-UNDO.
DEF VAR iStart              AS INT  NO-UNDO.
DEF VAR iEnd                AS INT  NO-UNDO.
DEF VAR iINDEX              AS INT  NO-UNDO.
DEF VAR iFIELD              AS INT  NO-UNDO.
DEF VAR iLIKE               AS INT  NO-UNDO.

/* Export to Excel variables */
DEF VAR chExcelApp    AS COM-HANDLE NO-UNDO. 
DEF VAR chWorkbook    AS COM-HANDLE NO-UNDO. 
DEF VAR chWorksheet   AS COM-HANDLE NO-UNDO. 
DEF VAR cSheetName    AS CHAR       NO-UNDO.
DEF VAR iSheetRow     AS INT        NO-UNDO.
DEF VAR iSheetCount   AS INT        NO-UNDO.
