/* ******************************************** */
/* Program   : IDX004.I                         */
/* Purpose   : XREF-Analyse procedure           */
/* Parameters: &BatchMode : True/False          */
/* ******************************************** */
  
  FOR EACH ttXRef WHERE ttXRef.xRefType = "SEARCH":u 
                    AND ttXRef.xMisc1  <> "RECID":u  NO-LOCK BY ttXref.LineId:
      /** debugging
      OUTPUT STREAM stDebug TO "c:\progress\chkidx\debug\ttDEBUG.txt" APPEND.
      EXPORT STREAM stDebug ttXRef.
      OUTPUT STREAM stDebug CLOSE.
      debugging **/
     ASSIGN cXRefDatabaseName = IF ttXRef.xMisc2 <> "TEMPTABLE":u
                                THEN ENTRY(1,ttXref.xObjectId,".":u)
                                ELSE "TEMPTABLE":u
            cXRefTableName    = IF ttXRef.xMisc2 <> "TEMPTABLE":u
                                THEN ENTRY(2,ttXref.xObjectId,".":u)
                                ELSE ttXref.xObjectId
            cXRefIndexName    = ttXRef.xMisc1.
     
     /* Analyse SEARCH statement ---------------------------------- */
     IF AVAIL ttSearch AND ttSearch.DatabaseName = cXRefDatabaseName
                       AND ttSearch.TableName    = cXRefTableName
                       AND ttSearch.XRef-LineId  = ttXRef.LineId - 1
     THEN ASSIGN ttSearch.UsedIndexes = ttSearch.UsedIndexes + ",":u + cXRefIndexName
                 ttSearch.XRef-LineId = ttXRef.LineId.
     ELSE DO:
        CREATE ttSearch.
        ASSIGN ttSearch.Xref-LineId  = ttXRef.LineId
               ttSearch.DatabaseName = cXRefDatabaseName
               ttSearch.TableName    = cXRefTableName
               ttSearch.UsedIndexes  = cXRefIndexName
               ttSearch.LineId       = iSearchLineId
               iSearchLineId         = iSearchLineId + 1.

       &IF NOT {&BatchMode} &THEN
        IF LOOKUP(ttSearch.TableName,CB-Table:LIST-ITEMS) = 0 AND
           (ttSearch.DatabaseName <> "TEMPTABLE":u OR MENU-ITEM m_Analyse_Temptables:CHECKED IN MENU m_Options)
        THEN CB-Table:ADD-LAST(ttSearch.TableName).
       &ENDIF

        /* Look for SEARCH Type -------------------------------------- */
       &IF {&BatchMode} &THEN
        INPUT STREAM stFile FROM VALUE(ttXRef.xFileName) NO-ECHO.
        iLooper = 1.
        DO WHILE iLooper < ttXRef.xLineNumber :
           IMPORT STREAM stFile UNFORMATTED cVoid NO-ERROR.
           iLooper = iLooper + 1. 
        END. /* first loop */
        IMPORT STREAM stFile UNFORMATTED cVoid NO-ERROR.
        ttSearch.SearchType = IF INDEX(cVoid,"FIND":u) > 0
                              THEN "find":u ELSE "for":u.
        INPUT STREAM stFile CLOSE.
       &ELSE
        RUN ED-FileImport (INPUT ttXRef.xFileName
                          ,INPUT ttXRef.xLineNumber).
        ttSearch.SearchType = IF INDEX(ED-File:SELECTION-TEXT,"FIND":u) > 0
                              THEN "find":u ELSE "for":u.
       &ENDIF
        
        /* Look for used fields -------------------------------------- */
        ASSIGN lTableTagMeet  = FALSE
               lTableTagExist = CAN-FIND(FIRST bttXRef WHERE bttXRef.xProcName   = ttXRef.xProcName
                                                         AND bttXRef.xFileName   = ttXRef.xFileName
                                                         AND bttXRef.xLineNumber = ttXRef.xLineNumber
                                                         AND bttXRef.xRefType    = "STRING":u
                                                         AND bttXRef.xObjectId   = ttSearch.TableName                              
                                                         AND bttXRef.xMisc1      = TRIM(STRING(LENGTH(ttSearch.TableName),">9":u)) 
                                                         AND bttXRef.xMisc2      = "NONE":u                                        
                                                         AND bttXRef.xMisc3      = "UNTRANSLATABLE":u).
        FOR EACH bttXRef WHERE bttXRef.xProcName   = ttXRef.xProcName
                           AND bttXRef.xFileName   = ttXRef.xFileName
                           AND bttXRef.xLineNumber = ttXRef.xLineNumber NO-LOCK:
           ASSIGN lTableTagMeet = lTableTagMeet OR (bttXRef.xRefType  = "STRING":u
                                                AND bttXRef.xObjectId = ttSearch.TableName                              
                                                AND bttXRef.xMisc1    = TRIM(STRING(LENGTH(ttSearch.TableName),">9":u)) 
                                                AND bttXRef.xMisc2    = "NONE":u                                        
                                                AND bttXRef.xMisc3    = "UNTRANSLATABLE":u)
                  ttSearch.AccessedFields = ttSearch.AccessedFields + (IF (NOT lTableTagExist OR lTableTagMeet) 
                                                                          AND bttXRef.xRefType  = "ACCESS":u 
                                                                          AND bttXRef.xObjectId = ttXref.xObjectId /* same DB.Table */
                                                                          AND LOOKUP(bttXRef.xMisc1,ttSearch.AccessedFields) = 0 
                                                                       THEN ",":u + bttXRef.xMisc1 ELSE "") .
        END. /* each bttXRef */
        ASSIGN ttSearch.AccessedFields    = SUBSTRING(ttSearch.AccessedFields,2) /* skip first comma */
              &IF NOT {&BatchMode} &THEN
               FI-WholeIndex:HIDDEN       = FI-WholeIndex:HIDDEN AND NOT (ttXRef.xMisc2 = "WHOLE-INDEX":u AND NUM-ENTRIES(ttSearch.AccessedFields) > 0)
               FI-WholeIndex:PRIVATE-DATA = STRING(FI-WholeIndex:HIDDEN)
              &ENDIF.
     END. /* not avail ttSearch */

     /* Analyze INDEX used ---------------------------------------- */
     IF ttSearch.DatabaseName = "TEMPTABLE":u &IF NOT {&BatchMode} 
                                              &THEN AND MENU-ITEM m_Analyse_Temptables:CHECKED IN MENU m_Options 
                                              &ELSE AND piAnalyseTempTable
                                              &ENDIF
     THEN RUN IdxOnTempTable-Analyse (INPUT  ttXRef.xObjectId
                                     ,INPUT  cXRefIndexName).
     ELSE RUN IdxOnRealTable-Analyse (INPUT  ttSearch.DatabaseName
                                     ,INPUT  ttSearch.TableName
                                     ,INPUT  cXRefIndexName
                                     ,INPUT  ttSearch.DatabaseName
                                     ,INPUT  ttSearch.TableName
                                     ,OUTPUT lVoid).

     /* Score Calculation ----------------------------------------- */
     ASSIGN cFldMask          = ""
            dScore            = 0
            iNumIndexedFields = 0.
     IF INDEX(ttSearch.SearchType,"find":U)     > 0  OR 
        INDEX(ttSearch.SearchType,"can-find":U) > 0
     THEN DO:
        /* on FIND and CAN-FIND Searches, if a field part of the used index is missing
           in the Where Clause... then the Score is stopped */
        FOR EACH ttIndex WHERE ttIndex.DatabaseName = ttSearch.DatabaseName
                           AND ttIndex.TableName    = ttSearch.TableName
                           AND ttIndex.IndexName    = ttSearch.UsedIndexes NO-LOCK BY ttIndex.FieldSeq:
           SUBSTRING(cFldMask,ttIndex.FieldSeq,1) = IF LOOKUP(ttIndex.FieldName,ttSearch.AccessedFields) > 0
                                                    THEN "1":u ELSE "0":u.
        END. /* each ttIndex */
        ASSIGN iWorker = INDEX(cFldMask,"0":u)
               dScore  = IF iWorker > 0 AND INDEX(SUBSTRING(cFldMask,iWorker + 1),"1":u) > 0
                         THEN (iWorker - 1)
                         ELSE R-INDEX(cFldMask,"1":u).
     END. /* SearchType is FIND or CAN-FIND */
     ELSE DO iRunner = 1 TO NUM-ENTRIES(ttSearch.AccessedFields):
        /* In the following CAN-FIND, I have to choose between EQUALITY and BEGINS  
           - Using Equality, I miss the fields that are not fully named
           - Using Begins  , I double-count the fields that have quite the same name */
        dScore = dScore + (IF CAN-FIND(FIRST ttIndex WHERE ttIndex.DatabaseName = ttSearch.DatabaseName
                                                       AND ttIndex.TableName    = ttSearch.TableName
                                                       AND LOOKUP(ttIndex.IndexName,ttSearch.UsedIndexes) > 0
                                                       AND ttIndex.FieldName    = ENTRY(iRunner,ttSearch.AccessedFields))
                           THEN 1 ELSE 0).
     END. /* SearchType is FOR */
     /* Count indexed fields for Score 2 */
     FOR EACH ttIndex WHERE ttIndex.DatabaseName = ttSearch.DatabaseName
                        AND ttIndex.TableName    = ttSearch.TableName
                        AND LOOKUP(ttIndex.IndexName,ttSearch.UsedIndexes) > 0 NO-LOCK:
        iNumIndexedFields = iNumIndexedFields + 1.
     END. /* Each ttIndex */
     /* Format scores */
     ASSIGN ttSearch.IndexScore1  = dScore / NUM-ENTRIES(ttSearch.AccessedFields)
            ttSearch.IndexScore1  = (IF ttSearch.IndexScore1 = ? THEN 100 ELSE ttSearch.IndexScore1 * 100)
            ttSearch.IndexScore2  = dScore / iNumIndexedFields
            ttSearch.IndexScore2  = (IF ttSearch.IndexScore2 = ? THEN 100 ELSE ttSearch.IndexScore2 * 100)
           &IF NOT {&BatchMode} &THEN
            RE-Wait:WIDTH-PIXELS  = MIN(200,RE-Wait:WIDTH-PIXELS + ROUND(200 / iSearchStats,0))
           &ENDIF.
  END. /* for each ttXRef */
