/* ******************************************** */
/* Program   : IDX006.I                         */
/* Purpose   : IdxOnTempTable-Analyse procedure */
/* Parameters: none                             */
/* ******************************************** */

  DEF INPUT PARAM piTableName   AS CHAR NO-UNDO.
  DEF INPUT PARAM piIndexName   AS CHAR NO-UNDO.


  FOR FIRST ttTempTable FIELDS(TTDefinition)
                        WHERE ttTempTable.TTName = piTableName NO-LOCK: END.
  IF AVAIL ttTempTable
  THEN cLine = ttTempTable.TTDefinition.
  ELSE DO:
     /* Look for Temp-Table definition ----------------------------------- */
     FIND FIRST bttXRef WHERE bttXRef.xRefType  = "STRING":u
                          AND bttXRef.xObjectId = piTableName
                        USE-INDEX i1 NO-LOCK NO-ERROR.
     ASSIGN cLine            = ""
            cProcName        = bttXRef.xProcName
            cFileName        = bttXRef.xFileName
            iLineNumber-from = bttXRef.xLineNumber.
     FIND NEXT  bttXRef WHERE bttXRef.xProcName   = cProcName
                          AND bttXRef.xFileName   = cFileName
                          AND bttXRef.xLineNumber > iLineNumber-from
                        USE-INDEX i1 NO-LOCK NO-ERROR.
     ASSIGN iLineNumber-to   = (IF AVAIL bttXRef
                                THEN MAX(iLineNumber-from,bttXRef.xLineNumber - 2)
                                ELSE iLineNumber-from + 9999)
            iLooper          = 1
            lVoid            = FALSE.

     /* Import Temp-Table definition into one string --------------------- */
     INPUT STREAM stFile FROM VALUE(cFileName) NO-ECHO.
     DO WHILE iLooper < iLineNumber-from :
        IMPORT STREAM stFile UNFORMATTED cVoid NO-ERROR.
        iLooper = iLooper + 1. 
     END. /* first loop */
     SecondLoop:
     DO WHILE iLooper <= iLineNumber-to AND NOT lVoid ON ENDKEY UNDO SecondLoop, LEAVE SecondLoop :
        IMPORT STREAM stFile UNFORMATTED cVoid NO-ERROR.
        ASSIGN lVoid   = lVoid OR ERROR-STATUS:ERROR OR cVoid = "/* _UIB-CODE-BLOCK-END */":u 
               cLine   = (IF NOT lVoid 
                          THEN TRIM(cLine) + " ":u + cVoid
                          ELSE cLine)
               iLooper = iLooper + 1.
     END. /* second loop */
     INPUT STREAM stFile CLOSE.
  
     /* Remove commented strings */
     DO WHILE INDEX(cLine,"/*":u) > 0:
        ASSIGN iStart = INDEX(cLine,"/*":u)
               iEnd   = INDEX(cLine,"*/":u,iStart)
               cLine  = SUBSTRING(cLine,1,iStart - 1) + SUBSTRING(cLine,iEnd + 2).
     END.
     /* Remove unnecessary keywords,double blank and final dot */
     ASSIGN cLine = REPLACE(cLine," PRIMARY ":u," ":u)
            cLine = fct_RemoveString(cLine," UNIQUE ":u)
            cLine = fct_RemoveString(cLine," IS ":u)
            cLine = fct_RemoveString(cLine,"  ":u)
            cLine = IF SUBSTRING(cLine,LENGTH(cLine),1) = ".":u
                    THEN SUBSTRING(cLine,1,LENGTH(cLine) - 1)
                    ELSE cLine.

     /* Store TempTable definition */
     CREATE ttTempTable.
     ASSIGN ttTempTable.LineId        = iTempTableLineId
            ttTempTable.TTName        = piTableName
            ttTempTable.TTDefinition  = cLine
            ttTempTable.TTDBReference = ""
            iTempTableLineId          = iTempTableLineId + 1.
  END. /* not avail ttTempTable */

  /* Analyze Temp-Table definition ------------------------------------ */
  ASSIGN /* add a final blank */
         cLine = cLine + " ":u
         /* move cursor just after temp-table name */
         cLine = SUBSTRING(cLine,INDEX(cLine,piTableName) + LENGTH(piTableName))
         /* Look for hardcoded index definition */
         iINDEX = INDEX(cLine," INDEX ":u + piIndexName).

  IF iINDEX > 0 AND NOT CAN-FIND(FIRST ttIndex WHERE ttIndex.DatabaseName = "TEMPTABLE":u
                                                 AND ttIndex.TableName    = piTableName
                                                 AND ttIndex.IndexName    = piIndexName)
  THEN DO:
     /* Used index is hardcoded in temp-table definition */
     ASSIGN cLine      = SUBSTRING(cLine,iINDEX + LENGTH(" INDEX ":u + piIndexName))
            iEnd       = INDEX(cLine," INDEX ":u)
            iEnd       = IF iEnd = 0 THEN LENGTH(cLine) ELSE iEnd
            cFieldList = TRIM(SUBSTRING(cLine,1,iEnd))
            cFieldList = REPLACE(cFieldList,".":u,"").
     FIND LAST ttIndex NO-LOCK NO-ERROR.
     ASSIGN iIndexLineId = IF AVAIL ttIndex THEN (ttIndex.LineId + 1) ELSE 1
            iFieldSeq    = 1  .
     DO iLooper = 1 TO NUM-ENTRIES(cFieldList," ":u):
        IF NOT ENTRY(iLooper,cFieldList," ":u) BEGINS "ASC":u AND
           NOT ENTRY(iLooper,cFieldList," ":u) BEGINS "DESC":u
        THEN DO:
           CREATE ttIndex.
           ASSIGN ttIndex.LineId       = iIndexLineId
                  ttIndex.DatabaseName = ttSearch.DatabaseName
                  ttIndex.TableName    = ttSearch.TableName
                  ttIndex.IndexName    = piIndexName
                  ttIndex.FieldName    = ENTRY(iLooper,cFieldList," ":u)
                  ttIndex.FieldSeq     = iFieldSeq
                  ttIndex.SortOrder    = IF iLooper < NUM-ENTRIES(cFieldList," ":u) AND ENTRY(iLooper + 1,cFieldList," ":u) BEGINS "DESC":u
                                         THEN "D":u ELSE "A":u
                  ttIndex.IsUnique     = lIsUnique
                  iIndexLineId         = iIndexLineId + 1
                  iFieldSeq            = iFieldSeq + 1.
        END. /* entry is not ASC nor DESC */
     END. /* iLooper */
  END. /* Hardcoded index */
  ELSE DO:
     /* Look for LIKE keyword... before any FIELD keyword */
     ASSIGN iLIKE  = INDEX(cLine,"LIKE":u)
            iFIELD = INDEX(cLine,"FIELD":u).
  
     IF iLIKE > 0 AND (iFIELD = 0 OR iFIELD > iLIKE)
     THEN DO:
        /* temp-table structure is inherited from a DB table */
        ASSIGN cLine      = SUBSTRING(cLine,iLIKE + 5)
               cTableName = SUBSTRING(cLine,1,INDEX(cLine," ":u))
               iRunner    = 1
               lVoid      = FALSE.
       
        /* Look for used index in DB table structure */
        DO WHILE iRunner <= NUM-DBS AND NOT lVoid:
           RUN IdxOnRealTable-Analyse (INPUT  LDBNAME(iRunner)
                                      ,INPUT  cTableName
                                      ,INPUT  piIndexName
                                      ,INPUT  ttSearch.DatabaseName
                                      ,INPUT  ttSearch.TableName
                                      ,OUTPUT lVoid).
           /* Save DB reference */
           ASSIGN ttTempTable.TTDBReference = IF lVoid THEN LDBNAME(iRunner) ELSE ttTempTable.TTDBReference
                  iRunner                   = iRunner + 1.
        END.
     END. /* Like keyword found */
  END. /* not hardcoded index */
