/* ************************************************ */
/* Program   : IDXBAT.P                             */
/* Purpose   : Perform Index Check in Batch Mode    */
/* Parameters: new                                  */
/* ************************************************ */

DEF INPUT PARAM piInputDirectory    AS CHAR NO-UNDO.
DEF INPUT PARAM piFileMask          AS CHAR NO-UNDO.
DEF INPUT PARAM piOutputTo          AS CHAR NO-UNDO.
DEF INPUT PARAM piSortOrder         AS INT  NO-UNDO.
DEF INPUT PARAM piMaxScore          AS DEC  NO-UNDO.
DEF INPUT PARAM piAnalyseTempTable  AS LOG  NO-UNDO.
    
/* TempTables and Variables definitions ----------- */              
{incl\idx001.i NEW}
{incl\idx003.i}

/* Working variables */
DEF VAR cSeparator      AS CHAR NO-UNDO.
DEF VAR cPath           AS CHAR NO-UNDO.
DEF VAR cMask           AS CHAR NO-UNDO.

/* User-defined Function -------------------------- */ 
{incl\idx007.i}

/* Main Block ------------------------------------- */

/* Test input parameters */
ASSIGN piFileMask         = IF piFileMask = ? OR piFileMask = ""
                            THEN "*.xrf":u ELSE piFileMask
       piMaxScore         = IF piMaxScore = ? OR piMaxScore = 0
                            THEN 100       ELSE piMaxScore
       piAnalyseTempTable = IF piAnalyseTempTable = ?
                            THEN TRUE ELSE piAnalyseTempTable.

/* Prepare Excel session */
IF piOutputTo = "excel":u
THEN RUN Excel-Open (INPUT-OUTPUT piOutputTo). 

/* Look for XRef Files */
ASSIGN cSeparator = (IF INDEX(piInputDirectory,"/":u) > 0
                     THEN "/":u ELSE "\":u)
       cPath      = piInputDirectory
       cPath      = cPath + (IF SUBSTRING(cPath,LENGTH(cPath),1) = cSeparator
                             THEN "" ELSE cSeparator)
       cMask      = cPath + piFileMask.

RUN FileFindLoop (cMask                   /* mask, must contain wildcards */
                 ,"ProcessOneFile":u      /* name of callback procedure   */
                 ,THIS-PROCEDURE:HANDLE). /* location of callback proc    */

/* Make Excel session visible */
IF piOutputTo = "Excel":u
THEN RUN Excel-close.

/* External Procedures ---------------------------- */
&GLOB  FIND_DATA-SIZE 4           /* dwFileAttributes       */~
                    + 8           /* ftCreationTime         */~
                    + 8           /* ftLastAccessTime       */~
                    + 8           /* ftLastWriteTime        */~
                    + 4           /* nFileSizeHigh          */~
                    + 4           /* nFileSizeLow           */~
                    + 4           /* dwReserved0            */~
                    + 4           /* dwReserved1            */~
                    + 260         /* cFileName[MAX_PATH]    */~
                    + 14          /* cAlternateFileName[14] */

PROCEDURE FindFirstFileA EXTERNAL "kernel32":u :
  DEFINE INPUT  PARAMETER lpFileName      AS CHAR.
  DEFINE INPUT  PARAMETER lpFindFileData  AS MEMPTR.
  DEFINE RETURN PARAMETER hSearch         AS LONG.
END PROCEDURE.    

PROCEDURE FindNextFileA EXTERNAL "kernel32":u :
  DEFINE INPUT  PARAMETER hSearch         AS LONG.
  DEFINE INPUT  PARAMETER lpFindFileData  AS MEMPTR.
  DEFINE RETURN PARAMETER found           AS LONG.
END PROCEDURE.

PROCEDURE FindClose EXTERNAL "kernel32":u :
  DEFINE INPUT  PARAMETER hSearch         AS LONG.
END PROCEDURE.

/* Internal Procedures ---------------------------- */
/* -----------------------------------------------------------------
   procedure FileFindLoop
   purpose   want to scan a directory using wildcards?
             call FileFindLoop; for each found file it will run
             value(ipProcessFindData) in hExtProc. That callback-
             procedure can use the lpFindData structure to decide
             what to do next.
   ----------------------------------------------------------------- */
PROCEDURE FileFindLoop :
  DEF INPUT PARAM FileMask          AS CHAR.
  DEF INPUT PARAM ipProcessFindData AS CHAR.
  DEF INPUT PARAM hExtProc          AS HANDLE.
   
  DEF VAR lpFindData    AS MEMPTR   NO-UNDO.
  DEF VAR hSearch       AS INT      NO-UNDO.
  DEF VAR found         AS INT      NO-UNDO INIT 1.

  SET-SIZE(lpFindData) = {&FIND_DATA-SIZE}.

  RUN FindFirstFileA (FileMask, lpFindData, OUTPUT hSearch).
  IF hSearch <> -1
  THEN DO:
     DO WHILE found <> 0 :
        RUN VALUE(ipProcessFindData) IN hExtProc (lpFindData).
        RUN FindNextFileA (hSearch, lpFindData, OUTPUT found).
     END.
     RUN FindClose (hSearch).
  END.

  SET-SIZE(lpFindData) = 0.
END PROCEDURE.

PROCEDURE ProcessOneFile :
  DEF INPUT PARAM lpFindData    AS MEMPTR.
 
  DEF VAR cOutputFile   AS CHAR NO-UNDO.
  DEF VAR clongname     AS CHAR NO-UNDO.
  
  RUN FileInfo_LongName (lpFindData, OUTPUT clongname).
  RUN xref-Import (cPath + cLongName).
  RUN xref-Analyse.
  cOutputFile = IF piOutputTo = "excel":u
                THEN cLongName
                ELSE cPath + SUBSTRING(cLongName,1,R-INDEX(cLongName,".":u)) + "cix":u.
  RUN xref-Export (INPUT piOutputTo
                  ,INPUT cOutputFile
                  ,INPUT piSortOrder
                  ,INPUT piMaxScore).
END PROCEDURE.
 
PROCEDURE FileInfo_LongName :
  DEF INPUT  PARAMETER lpFindData AS MEMPTR.
  DEF OUTPUT PARAMETER cFileName  AS CHAR INIT "".

  IF GET-SIZE(lpFindData)={&FIND_DATA-SIZE} 
  THEN cFileName = GET-STRING(lpFindData, 45). /* =cFileName */
END PROCEDURE.

PROCEDURE idxOnRealTable-Analyse:
  {incl\idx005.i}
END PROCEDURE.

PROCEDURE idxOnTempTable-Analyse:
  {incl\idx006.i}
END PROCEDURE.
    
PROCEDURE xref-import:
  {incl\idx008.i}
END PROCEDURE.

PROCEDURE xref-analyse:
  {incl\idx004.i &BatchMode=TRUE}
END PROCEDURE.

PROCEDURE TT-delete:
  {incl\idx009.i}
END PROCEDURE.

PROCEDURE Excel-Open:
  {incl\idx010.i}
END PROCEDURE.

PROCEDURE Excel-Close:
  {incl\idx011.i}
END PROCEDURE.

PROCEDURE xref-export:
  {incl\idx012.i}
END PROCEDURE. 

PROCEDURE xref-Export-Data:
  {incl\idx013.i}
END PROCEDURE. /* Export-data */
