/* ******************************************** */
/* Program   : CHKIDX-Tester.P                  */
/* Purpose   : Test several cases of SEARCH     */
/* Parameters: none                             */
/* ******************************************** */

DEF NEW SHARED TEMP-TABLE ttTable1 NO-UNDO
    FIELD   test1   AS CHAR
    FIELD   test2   AS CHAR
    FIELD   test3   AS CHAR
    FIELD   test4   AS CHAR
    FIELD   test5   LIKE customer.NAME
    INDEX   tti1   IS PRIMARY UNIQUE test1 ASC test2 DESC
    INDEX   tti2   IS UNIQUE test2
    INDEX   tti3   test4 
                   test3 
                   test2 DESC 
                   test1.
 
DEF NEW SHARED TEMP-TABLE ttTable2 NO-UNDO
    LIKE customer
    USE-INDEX NAME
    INDEX city city.

DEF NEW SHARED TEMP-TABLE ttTable3 NO-UNDO
    FIELD FldKey    AS INT
    FIELD Fld01     AS CHAR
    FIELD Fld02     AS CHAR
    FIELD Fld03     AS CHAR
    FIELD Fld04     AS CHAR
    FIELD Fld05     AS CHAR
    FIELD Fld06     AS CHAR
    FIELD Fld07     AS CHAR
    FIELD Fld08     AS CHAR
    FIELD Fld09     AS CHAR
    FIELD Fld10     AS CHAR
    FIELD Fld11     AS CHAR
    FIELD Fld12     AS CHAR
    FIELD Fld13     AS CHAR
    FIELD Fld14     AS CHAR
    FIELD Fld15     AS CHAR
    FIELD Fld16     AS CHAR
    FIELD Fld17     AS CHAR
    FIELD Fld18     AS CHAR
    FIELD Fld19     AS CHAR
    FIELD Fld20     AS CHAR
    INDEX Idx01 IS PRIMARY UNIQUE FldKey    /* some comments */
    INDEX Idx02 Fld01 Fld02 Fld03 Fld04 
    /* other comments */
    INDEX Idx03 Fld05 Fld06 Fld07 Fld08 
    INDEX Idx04 Fld09 Fld10 Fld11 Fld12
    INDEX Idx05 Fld13 Fld14 Fld15 Fld16
    INDEX Idx06 Fld17 Fld18 Fld19 Fld20.

/* Temp-table - Multiple indexes ********************** */
FOR EACH ttTable3 WHERE ttTable3.FLd01 = "A"
                    AND ttTable3.FLd02 = "A"
                    AND ttTable3.FLd03 = "A"
                    AND ttTable3.FLd04 = "A"
                    AND ttTable3.FLd05 = "A"
                    AND ttTable3.FLd06 = "A"
                    AND ttTable3.FLd07 = "A"
                    AND ttTable3.FLd08 = "A"
                    AND ttTable3.FLd09 = "A"
                    AND ttTable3.FLd10 = "A"
                    AND ttTable3.FLd11 = "A"
                    AND ttTable3.FLd12 = "A"
                    AND ttTable3.FLd13 = "A"
                    AND ttTable3.FLd14 = "A"
                    AND ttTable3.FLd15 = "A"
                    AND ttTable3.FLd16 = "A"
                    AND ttTable3.FLd17 = "A"
                    AND ttTable3.FLd18 = "A"
                    AND ttTable3.FLd19 = "A"
                    AND ttTable3.FLd20 = "A" NO-LOCK:
    MESSAGE "found" VIEW-AS ALERT-BOX.
END.

/* Normal Whole-Index (not highlighted) *************** */
FOR EACH order NO-LOCK:
END.

/* Temp-table - bad index ***************************** */
FIND FIRST ttTable1 WHERE ttTable1.test2 = "A":u NO-LOCK USE-INDEX tti1 NO-ERROR.
FIND FIRST ttTable1 WHERE ttTable1.test2 = "A":u NO-LOCK USE-INDEX tti2 NO-ERROR.
FIND FIRST ttTable1 WHERE ttTable1.test2 = "A":u NO-LOCK USE-INDEX tti3 NO-ERROR.

/* Temp-table - index inherited from real table ******* */
FIND FIRST ttTable2 WHERE ttTable2.NAME BEGINS "A":u NO-LOCK NO-ERROR.

/* Temp-table - hardcoded index *********************** */
FIND FIRST ttTable2 WHERE ttTable2.city = "bruxelles":u NO-LOCK NO-ERROR.

/* Real table - multiple indexes ********************** */
FOR EACH customer WHERE customer.country     = "Finland":u
                    AND customer.postal-code = "1234":u
                    AND customer.NAME        = "bumm bumm tennis":u
                    AND customer.sales-rep   = "SLS":u NO-LOCK:
   MESSAGE "customer found" VIEW-AS ALERT-BOX.
END.

/* Real table - multiple indexes ********************** */
FOR EACH order WHERE order.order-date = 01/01/1999
                 AND order.sales-rep  = "SLS":u NO-LOCK:
END.

/* Real table - partially indexed ********************* */
FIND FIRST customer WHERE Customer.Postal-Code = "1234":u
                      AND Customer.NAME        = "bumm bumm tennis":u NO-LOCK NO-ERROR.

/* Real table - no index ****************************** */
FOR EACH ITEM WHERE Item.Item-Name BEGINS "squash":u
                AND Item.Price     > 1000 NO-LOCK:
END.

/* Real table - can find statement ******************** */
IF CAN-FIND(FIRST order WHERE Order.Cust-Num = 100) 
THEN MESSAGE "Found Order for customer 100" VIEW-AS ALERT-BOX.

/* Preprocess in FIND ********************************* */
&SCOPED-DEFINE TableName salesrep
&SCOPED-DEFINE FieldName Rep-Name
FIND FIRST {&TableName} WHERE {&TableName}.{&FieldName} = "Robert Roller":u NO-LOCK NO-ERROR.

/* Reference through Include-file ********************* */
{c:\progress\chkidx\test\incl001.i}

/* Embedded FOR EACHes ******************************** */
FOR EACH order WHERE order.order-num > 1000 NO-LOCK,
    EACH order-line OF order NO-LOCK:
END.

/* Sort option **************************************** */
FOR EACH order WHERE order.order-num > 1000 NO-LOCK BY order-date:
END.

/* Embedded test ************************************** */
FIND FIRST ref-call NO-LOCK NO-ERROR.
IF ref-call.Call-date = TODAY
   AND CAN-FIND(FIRST ref-call WHERE ref-call.cust-num = 1
                                 AND ref-call.call-num = "1":u)
THEN TRUE.
